Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type JsonReader ¶
type JsonReader struct {
*io.FunctionalSensor
// contains filtered or unexported fields
}
A JsonReader is a type of FunctionalSensor where each call to evoke produces a new QualitativeSignal from the set of signal data based on the currently set
func NewJsonReader ¶
func NewJsonReader(signals io.JsonSignalArray, targetId string) *JsonReader
func (*JsonReader) SetTargetSignal ¶
func (jsR *JsonReader) SetTargetSignal(id string)
SetTargetSignal sets the id that should be evoked from the JsonSignalArray
type TextReader ¶
type TextReader struct {
io.FunctionalSensor
// contains filtered or unexported fields
}
A TextReader is a type of FunctionalSensor where each call to evoke produces a new QualitativeSignal for each letter in the TextReader's text property. Once the string has been read out, no more signals will be produced. Text is considered to be only the 26 letters of the alphabet. Text is not case-sensitive. Text which is not a letter is interpreted to be a blank signal.
func NewTextReader ¶
func NewTextReader(text string) *TextReader
NewTextReader creates a nex TextReader instance with the text value passed to text
func (*TextReader) Next ¶
func (tr *TextReader) Next() bool
Next returns true if there is any remaining text to be read
func (*TextReader) Reset ¶
func (tr *TextReader) Reset()
Reset returns the index value of the TextReader to 0 to begin evocations again.